"""world.py

  Author: David "Sofox" Collins
  Description: Geography Quiz game
  """
# GP2X joystick button mappings
BUTTON_UP            = (0)
BUTTON_DOWN          = (4)
BUTTON_LEFT          = (2)
BUTTON_RIGHT         = (6)
BUTTON_UPLEFT        = (1)
BUTTON_UPRIGHT       = (7)
BUTTON_DOWNLEFT      = (3)
BUTTON_DOWNRIGHT     = (5)
BUTTON_CLICK         = (18)
BUTTON_A             = (12)
BUTTON_B             = (13)
BUTTON_X             = (14)
BUTTON_Y             = (15)
BUTTON_L             = (10)
BUTTON_R             = (11)
BUTTON_START         = (8) 
BUTTON_SELECT        = (9)
BUTTON_VOLUP         = (16)
BUTTON_VOLDOWN       = (17)

#print "Path:"
import sys
#print repr(sys.path)

import pygame
from pygame.locals import *
import os
import pygame
import random

from pygame.locals import *

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

class quizQuestion:
    country=""
    options=["","","",""]
    correctAnswer=-1

class quiz:
    score=0
    questionNumber=1
    def startQuiz(self,tq=20):
        self.score=0
        self.questionNumber=1
        self.totalQuestions=tq
        self.lastAnswer=""
        self.lastCorrect=False
    def nextQuestion(self, correct,answer=""):
        self.questionNumber=self.questionNumber+1
        self.lastCorrect=correct
        self.lastAnswer=answer
        if correct:
            self.addPoint()
    def getLastAnswer(self):
        return self.lastAnswer
    def getLastCorrect(self):
        return self.lastCorrect
    def addPoint(self):
        self.score=self.score+1
    def getPoints(self):
        return self.score
    def isQuizOver(self):
        return self.questionNumber>self.totalQuestions
        
        
class gameState:
    def setup(self,s,wloc):
        self.screen=s
        self.state=5
        self.keyPressed=self.sp=self.hp=self.jp=self.kp=self.lp=False

        self.inp=[False,False,False,False]
        self.guy=pygame.image.load(wloc).convert()
        self.guyrect=self.guy.get_rect()

        self.font = pygame.font.Font(None,28)
        self.text = self.font.render("This is a geography quiz", 1, (255,255,255))
        self.textrect = self.text.get_rect()

        self.countryList = []
        self.capitalList = []
        self.contCapList = {}

        self.currQuiz= quiz()
        self.currQuiz.startQuiz()
        self.currQuestion=quizQuestion()
        self.debug=False
    def getText(self,s, colour=(255,255,255)):
        colour=(255,255,255)
        return self.font.render(s, 1, colour)
    def getScreen(self):
        return self.screen

    def newQuestion(self):
        self.currQuestion=self.makeQuestion()
        
    def clearInput(self):
        self.hp=self.jp=self.kp=self.lp=False
    def getQuestion(self):
        return str(self.currQuiz.questionNumber) + ". What is the capital of "+self.currQuestion.country+"?"
    def getQuestionTotal(self):
        return str(self.currQuiz.totalQuestions)
    def getOption(self,ind):
        return self.currQuestion.options[ind]
    def getAnswer(self,numeric=False):
        if numeric:
            return self.currQuestion.correctAnswer
        else:
            return str(self.currQuestion.correctAnswer)

    def getScore(self):
        return str(self.currQuiz.getPoints())
    def startQuiz(self):
        self.currQuiz.startQuiz()
        self.state=5
    def getLastAnswer(self):
        return self.currQuiz.getLastAnswer()
    def getLastCorrect(self):
        return self.currQuiz.getLastCorrect()

    def makeQuestion(self):
        question=quizQuestion()
        
        question.country=random.choice(self.countryList)
        realCap=self.contCapList[question.country]
        caplist=random.sample(self.capitalList,4)
        if realCap not in caplist:
            caplist[random.randint(0,3)]=realCap
        question.options=caplist
        question.correctAnswer=caplist.index(realCap)
        return question
    def step(self):
        if self.state==5:
            self.inp = [self.hp,self.jp,self.kp,self.lp]
            
            if True in self.inp:
                ans=self.inp.index(True)    #Getting the selected answer
                if self.getAnswer(True)==-1 or ans==-1: #Error catching
                    return;
                rightAnswer=self.getOption(self.getAnswer(True))
                self.currQuiz.nextQuestion(self.getAnswer(True)==ans,rightAnswer)
                if self.currQuiz.isQuizOver():
                    self.state=3
                else:
                    self.newQuestion()
        elif self.state==3:
            if self.keyPressed:
                self.startQuiz()
        
        self.clearInput()           #Clearing input
        self.keyPressed=False


def splitStrings(s,ln):
    if len(s)>ln:
        return [s[:ln],s[ln:]]
    else:
        return [s]



def draw(gs):
    blue=0,0,255
    darkblue=0,0,160
    border = 0,0,0
    red= 255,0,0
    green=0,255,0
    if  gs.screen:
        gs.screen.fill(blue)
        
        surf = pygame.Surface([320, 30])
        if not gs.getLastAnswer()=="":
            if gs.getLastCorrect(): areaColour=green
            else: areaColour=red
        else: areaColour=blue
        surf.fill(areaColour)
        gs.screen.blit(surf, (0,0))
        surf2=pygame.Surface([320,4])
        surf2.fill(border)
        gs.screen.blit(surf2, (0,28-surf2.get_height()/2))#(30-surf2.height()/2,0))

        qy=(35)
        ay=65
        if gs.state==0:
            gs.screen.blit(gs.text,(2,2))
        elif gs.state==5:   #Quiz mode!!!
            cq=gs.getText(gs.getQuestion())
            
            if cq.get_width()>=gs.screen.get_width():
                cq1=gs.getText(gs.getQuestion()[:23])
                cq2=gs.getText(gs.getQuestion()[23:])
                gs.screen.blit(cq1,(0,qy))
                gs.screen.blit(cq2,(0,qy+20))

            else:
                gs.screen.blit(cq,(0,qy))#
            spc="    "
            buttonMaps=[spc+"Y: ","A: ",spc*2+"B: ",spc+"X: "]
            for x in range(4):
                gs.screen.blit(gs.getText(buttonMaps[x]+gs.getOption(x)),(0, (qy+ay + x*20)))
            if gs.debug: gs.screen.blit(gs.getText(gs.getAnswer()),(0, (qy+50 + -1*30)))
            #Gives the right answer
            if not gs.getLastAnswer()=="": #Printing the last result
                if gs.getLastCorrect():
                    response= "Correct, the answer was "
                    col=(0,255,0)
                else:
                    response = "No, the answer was "
                    col=(255,0,0)
                response=response + gs.getLastAnswer()
                #response="gr"
                gs.screen.blit(gs.getText(response,col), (0, 0))
            gs.screen.blit(gs.getText(" "*46+"Score: " + gs.getScore()),(0, (90 + (4*30))))

        elif gs.state==3:
            if not gs.getLastAnswer()=="": #Printing the last result
                if gs.getLastCorrect():
                    response= "Correct, the answer was "
                    col=(0,255,0)
                else:
                    response = "No, the answer was "
                    col=(255,0,0)
                response=response + gs.getLastAnswer()
                #response="gr"
                gs.screen.blit(gs.getText(response,col), (0, 0))
            scoreTotal="You got " + gs.getScore() + " out of " + gs.getQuestionTotal()
            gs.screen.blit(gs.getText(scoreTotal),(0,qy))
            if gs.getScore()==gs.getQuestionTotal():
                gs.screen.blit(gs.getText("You know your Geography!"),(0,qy+20))
            gs.screen.blit(gs.getText("Press Start to quit"),(0,qy+40))
            gs.screen.blit(gs.getText("Press any other button to play again"),(0,qy+60))
            #gs.screen.blit(gs.getText(scoreTotal),(0,qy))
        else:
            gs.screen.blit(gs.guy,gs.guyrect)

        
        pygame.display.flip()

def parseList(gs):
    gs.countryList = []
    gs.capitalList = []
    gs.contCapList = {}
    cap=open("captials.txt")
    try:
        for line in cap:
            pos = line.find(" - ")
            gs.countryList.append(line[0:pos])
            gs.capitalList.append(line[pos+3:].strip())
            gs.contCapList[gs.countryList[-1]]=gs.capitalList[-1]
            #print line[0:pos], " - ", line[pos+3:]
    finally:
        cap.close()


def mainGameLoop():
    pygame.init()
    pygame.font.init()
    #screensize=sw,sh=320,240
    
    screen = pygame.display.set_mode([320, 240], 0, 16)

    gs=gameState()
    gs.setup(screen,"world.bmp")

    parseList(gs)
    gs.newQuestion()

    pygame.mouse.set_visible(0)
    num_joysticks = pygame.joystick.get_count()
    if num_joysticks > 0:
        stick = pygame.joystick.Joystick(0)
        stick.init() # now we will receive events for the joystick
    try:
        while 1:
            #for event in pygame.event.get():
            event = pygame.event.poll()
            if True:
                #print "got event"
                #print repr(event)
                if event.type==pygame.QUIT:
                    pygame.font.quit()
                    pygame.quit()
                    return
                elif event.type == KEYDOWN:
                    gs.keyPressed=True
                    if (event.key == K_ESCAPE):
                        pygame.font.quit()
                        pygame.quit()
                        return
                    elif (event.key == K_h):
                        gs.hp=True
                    elif (event.key == K_j):
                        gs.jp=True
                    elif (event.key == K_k):
                        gs.kp=True
                    elif (event.key == K_l):
                        gs.lp=True
                    elif (event.key == K_SPACE):
                        gs.sp=True
                #if event.type == MOUSEBUTTONDOWN:
                #    print pygame.mouse.get_pos()
                #    print gs.guy#.get_at(pygame.mouse.get_pos())[:3]
                #    pass
                elif event.type == JOYBUTTONDOWN:
                    gs.keyPressed=True
                    if event.button == BUTTON_Y:
                        gs.hp=True
                    elif event.button == BUTTON_A:
                        gs.jp=True
                    elif event.button == BUTTON_B:
                        gs.kp=True
                    elif event.button == BUTTON_X:
                        gs.lp=True
                    elif event.button == BUTTON_BUTTON_START:
                        pygame.font.quit()
                        pygame.quit()
                        return
                    else:
                        pass#print "unknown button pushed"
            gs.step()
            draw(gs)
            pygame.time.wait(50)
    except:
        print "Something bad happened"
        print event
        print "Unexpected error:", sys.exc_info()[0]
        
        pygame.quit()

    
mainGameLoop()
